define(['testsuite/qunit', 'jquery', 'de_epages/remotesearch/ui/suggest', '$ready!'], function (QUnit, $, de_epages) {

	var suggestData = function (url) {
			return {
				"atomLink": {
					"href": url,
					"type": "application/json",
					"rel": "self"
				},
				"products": [{
					"name": "Jack Wolfskin Blizzard Jacket",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/ho_40407/ho_40407_black_xs.jpg",
					"filter": 17150
				}, {
					"name": "Jack Wolfskin Blizzard Jacket",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/ho_40407/ho_40407_croft_xs.jpg",
					"filter": 17146
				}, {
					"name": "Jack Wolfskin Flexibility RT",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/ho_1112105010/ho_1112105010_xs.jpg",
					"filter": 17114
				}, {
					"name": "Berghaus Paclite Jacket - Men",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/be_40401/be_4040101_black_xs.jpg",
					"filter": 17129
				}, {
					"name": "Berghaus Paclite Jacket - Men",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/be_40401/be_40401_blue_xs.jpg",
					"filter": 17122
				}, {
					"name": "Berghaus Paclite Jacket - Women",
					"type": "product",
					"image": "/WebRoot/Store/Shops/DemoShop/Products/be_40402/be_40402_black_xs.jpg",
					"filter": 17140
				}],
				"manufacturers": [{
					"matches": 4,
					"name": "Berghaus",
					"filter": {
						"fieldName": "FacetValue_Manufacturer",
						"attributeValue": "Berghaus"
					}
				}, {
					"matches": 3,
					"name": "Jack Wolfskin",
					"filter": {
						"fieldName": "FacetValue_Manufacturer",
						"attributeValue": "Jack Wolfskin"
					}
				}],
				"categories": [{
					"matches": 6,
					"name": "Jackets",
					"filter": {
						"fieldName": "FacetValue_CategoryID",
						"attributeValue": "17238"
					}
				}, {
					"matches": 1,
					"name": "Tents",
					"filter": {
						"fieldName": "FacetValue_CategoryID",
						"attributeValue": "17244"
					}
				}]
			};
		},

		ajax = $.ajax;

	$.ajax = function( o ){
		if( /REST\/search/.test(o.url) ){
			var result = $.Deferred();

			result.resolveWith(o.context || result, [suggestData(o.url)]);

			return result;
		}
		else{
			return ajax(o);
		}
	};

	QUnit.module("de_epages/remotesearch/ui/suggest");

	QUnit.asyncTest("de_epages/remotesearch/ui/suggest", function() {

		var t = [],

			form = $("#search")
				.on("submit", function( event ){
					var hidden = elem.next();

					t.push({name: hidden.attr('name'), value: hidden.val()});

					event.preventDefault();
				}),

			elem = $('#search input')
				.remotesearchUiSuggest({
					suggestUrl: "/REST/search",
					showImages: false
				});

		elem.val("ja")
			.trigger("keydown")
			.trigger("keyup");

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box').length,
			1,
			"$('.de_epages-remotesearchUiSuggest-box').length"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box').is(':visible'),
			true,
			"$('.de_epages-remotesearchUiSuggest-box').is(':visible')"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box li').length,
			13,
			"$('.de_epages-remotesearchUiSuggest-box li').length"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box li.Separator').length,
			3,
			"$('.de_epages-remotesearchUiSuggest-box li.Separator').length"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box li.products').length,
			6,
			"$('.de_epages-remotesearchUiSuggest-box li.products').length"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box li.categories').length,
			2,
			"$('.de_epages-remotesearchUiSuggest-box li.categories').length"
		);

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box li.manufacturers').length,
			2,
			"$('.de_epages-remotesearchUiSuggest-box li.manufacturers').length"
		);

		elem.trigger("blur");

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box').is(':visible'),
			false,
			"$('.de_epages-remotesearchUiSuggest-box').is(':visible')"
		);

		elem.trigger("focus");

		QUnit.equal(
			$('.de_epages-remotesearchUiSuggest-box').is(':visible'),
			true,
			"$('.de_epages-remotesearchUiSuggest-box').is(':visible')"
		);

		elem.trigger({type:"keydown",keyCode:40})
			.trigger({type:"keyup",keyCode:40});

		QUnit.strictEqual(
			$('.de_epages-remotesearchUiSuggest-box li.ui-select').data('index'),
			0,
			"Selected index 0"
		);

		elem.trigger({type:"keydown",keyCode:40})
			.trigger({type:"keyup",keyCode:40});

		QUnit.strictEqual(
			$('.de_epages-remotesearchUiSuggest-box li.ui-select').data('index'),
			1,
			"Selected index 1 after arrow down"
		);

		elem.trigger({type:"keydown",keyCode:38})
			.trigger({type:"keyup",keyCode:38})
			.trigger({type:"keydown",keyCode:38})
			.trigger({type:"keyup",keyCode:38});

		QUnit.strictEqual(
			$('.de_epages-remotesearchUiSuggest-box li.ui-select').length,
			0,
			"Selected index 1 after arrow up"
		);

		elem.trigger({type:"keydown",keyCode:38})
			.trigger({type:"keyup",keyCode:38});

		QUnit.strictEqual(
			$('.de_epages-remotesearchUiSuggest-box li.ui-select').data('index'),
			9,
			"Selected index 9 after arrow up"
		);

		elem.trigger({type:"keydown",keyCode:38})
			.trigger({type:"keyup",keyCode:38})
			.trigger({type:"keydown",keyCode:38})
			.trigger({type:"keyup",keyCode:38});

		QUnit.strictEqual(
			$('.de_epages-remotesearchUiSuggest-box li.ui-select').data('index'),
			7,
			"Selected index 7 after arrow up (2x)"
		);

		$('.de_epages-remotesearchUiSuggest-box li.ui-select').trigger('mousedown');

		QUnit.deepEqual(
			t[0],
			{"name": "FacetValue_Manufacturer", "value": "Jack Wolfskin"},
			"Check value on click on manufacturer item"
		);

		elem.trigger({type:"keydown",keyCode:40})
			.trigger({type:"keyup",keyCode:40});
		$('.de_epages-remotesearchUiSuggest-box li.ui-select').trigger('mousedown');

		QUnit.deepEqual(
			t[1],
			{"name": "FacetValue_CategoryID", "value": "17238"},
			"Check value on click on category item"
		);

		$('.de_epages-remotesearchUiSuggest-box').hide();

		QUnit.start();
	});
});