/*globals define*/
/*jslint ass: true, unparam: true */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/inc/customizetype
 */
define("de_epages/product/inc/customizetype", [
    "jquery",

    "ep/ui/colortool",
    "de_epages/presentation/ui/uploader"
], function ($) {
    'use strict';

    return {
        color: function (objectId, selectors) {
            var colorTrigger = $(selectors.colorTrigger),
                colorList = $(selectors.colorList),
                colorItems,
                colorAddButton = $(selectors.colorAdd),
                colorTmpl = $(selectors.colorTemplate),
                colorMax,

                colorChange = function () {
                    colorTrigger.trigger({
                        type: 'change',
                        dojoconnect: true
                    });
                },

                colorInit = function () {
                    colorItems = colorList.children();
                    colorMax = colorItems.length >= 20;
                    colorAddButton.attr('disabled', colorMax);
                },

                colorRemove = function () {
                    $(this)
                        .closest(selectors.colorList + " > *")
                        .remove();

                    colorChange();
                    colorInit();
                },

                colorAdd = function () {
                    if (colorMax) {
                        return;
                    }

                    colorTmpl
                        .tmpl([{}])
                        .appendTo(colorList)
                        .find(selectors.colors)
                        .uiColortool({
                            orientation: "top right",
                            transparent: false,
                            autoOpen: true
                        })
                        .end()
                        .find(selectors.colorRemove)
                            .on("click", colorRemove)
                        .end();

                    colorChange();
                    colorInit();
                };

            // init interactive color handles
            $(selectors.colors).uiColortool({
                orientation: "top right",
                transparent: false
            });

            $(selectors.colorRemove)
                .on("click", colorRemove);

            colorAddButton
                .on("click", colorAdd);

            colorInit();
        },

        font: function (objectId, selectors) {
            var fontTrigger = $(selectors.fontTrigger),
                fontTmpl = $(selectors.fontTemplate),
                fontBlock = $(selectors.fontBlock),
                fontList = $(selectors.fontList),
                fontName = $(selectors.fontName),
                fontUpload = $(selectors.fontUpload),
                fontUploadData = {
                    ViewAction: 'JSONViewResponse',
                    ChangeAction: 'UploadCustomFont',
                    FontName: '',
                    ObjectID: objectId
                },

                fontInit = function () {
                    fontBlock.toggle(!!fontList.children().length);
                },

                fontChange = function () {
                    fontTrigger.trigger({
                        type: 'change',
                        dojoconnect: true
                    });
                },

                fontRemove = function () {
                    $(this).closest(selectors.fontList + " > *").remove();
                    fontChange();
                    fontInit();
                },

                fontAdd = function (data) {
                    if (data[0] && data[0].success) {
                        data = data[0] && data[0].success;
                        fontTmpl
                            .tmpl([data])
                            .find(selectors.fontRemove)
                                .on("click", fontRemove)
                            .end()
                            .appendTo(fontList);
                    }

                    fontName.val('');
                    fontUpload.prop("disabled", true);
                    fontChange();
                    fontInit();
                };

            // init interactive font handles
            fontUpload.presentationUiUploader({
                multiple: false,
                data: fontUploadData,
                callback: fontAdd
            });

            fontName.on("keyup", function () {
                var val = fontUploadData.FontName = $.trim(fontName.val()),
                    span;

                fontUpload.prop("disabled", (val.length) ? false : true);
                fontUpload.presentationUiUploader("option", "data", fontUploadData);

                // Dirty hack to prevent uplaod without font name
                span = fontUpload.data().uiPresentationUiUploader.custom;

                if (val.length) {
                    span.off("click.fontName");
                } else {
                    span.on("click.fontName", function (evt) {
                        evt.preventDefault();
                    });
                }

            }).trigger("keyup");

            $(selectors.fontRemove)
                .on("click", fontRemove);

            fontInit();
        }
    };
});
