/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/input-test',
    'jquery',
    'ep/ui/input',
    'util/support',

    'ep/ui/form',
    'jquery/metaparse'
], function (QUnit, fixture, $, ep, support) {

    QUnit.module("ep.ui.input", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("ep.ui.input", function () {

        $('.ep-js').metaparse();

        var input = ep('input:first');

        QUnit.equal(
            input.uiInput('Instance') instanceof $.Widget,
            true,
            "uiInput widget was created and instance of jQuery.Widget"
        );

        $.each(['_create', '_init', '_leave', '_stateChanged', '_tooltipInit', 'addClass', 'removeClass', '_change', '_changeAttr', '_setOption', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.uiInput.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        // text input
        QUnit.equal(
            input.uiInput('option', 'placeholder'),
            'placeholder',
            "Get placeholder of text input"
        );

        input.uiInput('option', 'placeholder', 'new placeholder');

        QUnit.equal(
            input.uiInput('option', 'placeholder'),
            'new placeholder',
            "Get placeholder of text input after it was changed"
        );

        input.uiInput('addClass', 'myTestClass');

        QUnit.equal(
            input.hasClass('myTestClass'),
            true,
            'text input has class "myTestClass"'
        );

        input.uiInput('removeClass', 'myTestClass');

        QUnit.equal(
            input.hasClass('myTestClass'),
            false,
            '"myTestClass" was removed from text input'
        );

        input
            .trigger('focusin')
            .val('myTestString')
            .trigger('focusout');

        QUnit.equal(
            input.val(),
            'myTestString',
            'Get value of text input after the value was changed'
        );

        input.uiInput('destroy');

        QUnit.equal(
            input.data('epUiInput'),
            undefined,
            'uiInput widget was destroyed'
        );

        // radio
        QUnit.equal(
            $(':radio').length,
            2,
            "$(':radio').length"
        );

        QUnit.equal(
            $(':radio:checked').length,
            0,
            "$(':radio:checked').length"
        );

        $(':radio:first').trigger('click');
        QUnit.equal(
            $(':radio:checked').length,
            1,
            "$(':radio:checked').length"
        );

        // checkbox
        QUnit.equal(
            $(':checkbox').length,
            3,
            "$(':checkbox').length"
        );

        QUnit.equal(
            $(':checkbox:checked').length,
            0,
            "$(':checkbox:checked').length"
        );

        $(':checkbox:first').trigger('click');
        QUnit.equal(
            $(':checkbox:checked').length,
            1,
            "$(':checkbox:checked').length"
        );

        // button without form-attribute
        QUnit.equal(
            $._data($('#buttonWithoutFormAttribute')[0], 'events').click,
            undefined,
            "$._data($('#buttonWithoutFormAttribute')[0], 'events')['click']"
        );

        // button with form-attribute
        // html5 browser
        if (support.inputFormAttr) {
            QUnit.equal(
                $._data($('#buttonWithFormAttribute')[0], 'events').click,
                undefined,
                "$._data($('#buttonWithFormAttribute')[0], 'events')['click']"
            );
        // non-html5 browser
        } else {
            var clickObject = $._data($('#buttonWithFormAttribute')[0], 'events').click;

            QUnit.equal(
                clickObject && clickObject.length,
                1,
                "$._data($('#buttonWithFormAttribute')[0], 'events')['click']"
            );
        }

        QUnit.start();
    });

});