/*jslint browser: true, nomen: true*/
/*global define*/
define('de_epages/chronopost/ui/pointrelais', [
    'jquery',
    'ep',
    'google/maps',
    '$dict!ep/dict',
    '$tmpl!./pointrelais',
    '$tmpl!./pointrelais-row',
    '$tmpl!./pointrelais-popup',
    'jquery/ui/widget',
    'jquery/metaparse',
    'ep/ajax',
    'ep/alert'
], function ($, ep, maps, epDict, tmpl, rowTmpl, popupTmpl) {

    'use strict';

    $.widget('ui.chronopostUiPointrelais', {
        options: {
            zipcode: '75000',
            relaisId: null,
            tUpdate: epDict.translate('{Update}')
        },

        _create: function () {
            var self = this,
                rendered = tmpl(this.options),
                tmplElements = rendered.tmplItem('elements');

            rendered
                .appendTo(this.element)
                .find('.ep-js')
                .metaparse();

            $.extend(this, tmplElements);

            self._update();

            this.update.on('click', function () {
                self._update();
            });
        },

        _setOption: function (name, value) {
            this._superApply(arguments);

            if (name === 'zipcode') {
                this.zipcode.val(value);
                this._update();
            }
        },

        _update: function () {
            var self = this;

            this.options.zipcode = this.zipcode.val();

            ep.ajax({ // AJAX request
                data: {
                    ObjectID: this.options.relaisId,
                    ViewAction: 'JSONViewChronopostLocations',
                    ZipCode: this.options.zipcode
                },
                cache: false,
                type: "POST",
                dataType: "json"
            }).done(function (data) {
                self.relaisData = data;
                self._updateMap();
                self._updateLocationTable();
            }).fail(function (errJSON) {
                var thisJSON = $.parseJSON(errJSON.responseText || "null"),
                    errMessage = thisJSON.Errors[0].Reason;

                ep.alert({
                    title: '{Error}',
                    type: 'warn',
                    content: errMessage
                });
            });
        },

        _updateMap: function () {
            //define map properties
            var self = this,
                // Fallback if no relais is available should be center of Paris
                firstRelais = this.relaisData[0] || {
                    latitude:  '47.9102777778',
                    longitude: '1.90805555556'
                },
                mapOptions = {
                    center: new maps.LatLng(firstRelais.latitude, firstRelais.longitude),
                    zoom: 13,
                    mapTypeId: maps.MapTypeId.ROADMAP
                },
                //initialize map
                googleMap = new maps.Map(this.map.get(0), mapOptions);

            $.each(this.relaisData, function (i, data) {
                var myLatLng = new maps.LatLng(data.latitude, data.longitude),
                    pointCHR = new maps.Marker({
                        position: myLatLng,
                        map: googleMap,
                        title: data.name
                    }),
                    addrDataPopup = [{
                        nameP: data.name,
                        addressP: data.address,
                        zipP: data.zipcode,
                        cityP: data.city
                    }],
                    contentString = popupTmpl(addrDataPopup),
                    infowindow = new maps.InfoWindow({
                        content: contentString.html()
                    });

                maps.event.addListener(pointCHR, 'click', function () {
                    // Close last open info window in map
                    if (self.lastInfowindow) {
                        self.lastInfowindow.close();
                    }
                    // Set current info window as last one
                    self.lastInfowindow = infowindow;
                    // Open info window
                    infowindow.open(googleMap, pointCHR);
                    // Select the radio button of the related info dialog
                    self._selectLocation(i);
                });
            });
        },

        _updateLocationTable: function () {
            this.locations.empty()
                .append(rowTmpl({addresses: this.relaisData}))
                .find('.ep-js')
                .metaparse();

            this._selectLocation(0);
        },

        _selectLocation: function (i) {
            this.locations.find('.location_' + i)
                .prop('checked', 'checked');
        },

        destroy: function () {
            this.element.empty();

            this._superApply(arguments);
        }
    });

    return de_epages;

});